"""Settings module

This module will also hold global vars indicating if a given module is present or not

    begin                : Sat Dec 14 2002
    copyright            : (C) 2002 by Ricardo Niederberger Cabral
    email                : nieder|at|mail.ru
    Time-stamp:            <03/05/17 09:32:02 rnc>
"""
## ***************************************************************************
## *                                                                         *
## *   This program is free software; you can redistribute it and/or modify  *
## *   it under the terms of the GNU General Public License as published by  *
## *   the Free Software Foundation; either version 2 of the License, or     *
## *   (at your option) any later version.                                   *
## *                                                                         *
## ***************************************************************************

from string import *
import ConfigParser
import sys,os
import Error

ConfigDefault = {
    "browse.showdotfiles":        "0",
    "browse.shownormalfiles":        "0",
    "browse.thumbsingleclick":    "0",
    "browse.showmetadialog":      "0",                        
    "browse.groupthreshold":      "50",
    "browse.dragpopup":           "1",
    "browse.askbeforehugethumb":      "1",
    "database.recursiveadd":      "1",
    "database.beep":               "0",
    "database.alwayssave":        "1",
    "database.autoopen":          "~/.imgseek/img-db.iqd",
    "database.autoscannew":          "0",    
    "database.hideprogress"    :  "1",
    "database.alwayssave"    :  "1",
    "view.mainwidth":             "700",
    "view.mainheight":            "600",
    "view.maintab":               "0",
    "view.browsetab":               "0",
    "view.vsplit1":               "170",
    "view.vsplit2":               "236",    
    "view.hsplit1":               "144",
    "view.hsplit2":               "144",            
    "view.slidedelay":            "3",
    "view.autofit":               "0",
    "view.recurseslide":           "1",
    "view.smoothscale":               "1",
    "view.ssplit1":               "226",
    "view.ssplit2":               "180",            
    "query.realtimesketch":       "1"            
    }

class Env:
    """This class will hold all enviroment settings, and global vars """
    def __init__(self,wnd=None):
        """ init environment. Wnd is a pointer to the main dialog """
        print "Checking environment..."
        # default stuff
        self.imgseekdatadir=""
        self.options={}                 # will hold all app preferences
        self.hasImMagick=0              # if imageMagick C++ support is being used.
        self.hasPIL =0                  # if PIL module was found.
        self.hasIPTC=0                  # if PIL iptc module was found
        self.hasExport=0                # if export (dsv/xml) modules where found
        self.hasJpegtran=None           # if jpegtran was found (lossless transforms). If non-null, will hold the full path to the executable
        self.verbose=0
        self.wnd=wnd                    # main dialog
        self.curdb=None                 # current db
        ### parse command line options
        # I could/should use getopt here, just like it's done on imgSeekCmd, but i don't want to import another module and harm startup time just to parse a few CLI opts.
        self.verbose=("-v" in sys.argv)
        ### Import some modules and see if they are available
        try:
            import imgdb
        except:
            Error.PrintTB("Warning: Unable to load the C++ extension \"imgdb.so\" module. Make sure you installed imgSeek correctly, and email any install bug to \"imgseek-devel@lists.sourceforge.net\".")        
        try:
            from qt import QT_VERSION_STR
        except:
            Error.PrintTB("You system doesn't seem to have PyQT installed. Please install it before running this application. See http://www.riverbankcomputing.co.uk/pyqt/download.php and http://imgseek.sourceforge.net/requirements.html")        
        try:
            if int(QT_VERSION_STR[0]) >=3:   # guess qt version.
                self.qtv=3
            else:
                self.qtv=2
        except:
            Error.PrintTB("Error detecting PyQt/Qt version on Settings module")
            self.qtv=3
        try:
            import IptcExtract
            self.hasIPTC=1
            self.log("IPTC extraction","detected")
        except:
            self.hasIPTC=0
            self.log("IPTC extraction","not found. Install PIL.")
        try:
            import xml.dom.minidom
            self.hasExport=1
            self.log("Export modules (dsv/xml)","detected")
        except:
            self.hasExport=0
            self.log("Export modules (dsv/xml)","not found. Make sure you have Python-XML (and related) modules installed.")
        try:
            import Image
            self.hasPIL=1
            self.log("Python Imaging Library","detected")
        except:
            self.log("Python Imaging Library","not found. Please install it if you want to enable related features. (Check http://imgseek.sf.net/requirements.html)")
            self.hasPIL=0

        ####### Detect supported image formats
        self.exif_ext=["jpeg","jpe","jpg"]
        self.iptc_ext=["tif","tiff","jpeg","jpe","jpg"]
        if self.qtv>=3:
            self.qt_ext=["jpeg","jpe","jpg","gif","bmp","png","xbm","pnm"]    
        else:
            self.qt_ext=["gif","bmp","png","xbm"]
        self.magick_ext=["tiff","tif","tiff","tga","targa","jpeg","jpe","jpg","gif","bmp","png","xbm","pnm",
        "art","avs","cgm","cmyk","cut","dcm","dcx","dib","dpx","ept","fig","fpx","gplt","gray","hpgl","html","ico","jbig","jp2","jpc","mat","miff","mono",
        "mng","otb","p7","pbm","pcd","pcx","pdf","pgm","pix","ppm","ps","rad","rgb","sct","sgi","sun","txt","xbm","xcf"]
        self.pil_ext=["jpeg","jpe","jpg","gif","bmp","png","xbm","pnm"]
        self.doc_ext=["txt","pdf","ps","htm","html"]

        # set supported extensions
        if imgdb.hasImageMagick():
            self.hasImMagick=1
            self.supported_ext=self.qt_ext+self.magick_ext
        else:
            self.hasImMagick=0
            self.magick_ext=[]
            self.supported_ext=self.qt_ext
        if self.hasPIL:
            self.supported_ext=self.supported_ext+self.pil_ext
        if self.verbose: self.log("Supported formats",str(self.supported_ext))

        # run some tests in order to detect run-time features, set paths, etc """
        #### guess imgSeekLib data dir
        self.imgseekdatadir = sys.exec_prefix + "/share/imgSeek/"
        if not os.path.exists(self.imgseekdatadir):
            self.imgseekdatadir=sys.exec_prefix+"/share/imgseek/"
        if not os.path.exists(self.imgseekdatadir):
            self.imgseekdatadir=sys.exec_prefix+"/share/imgSeekLib/"
        if not os.path.exists(self.imgseekdatadir):
            self.imgseekdatadir=sys.exec_prefix+"/imgSeekLib/"
        if not os.path.exists(self.imgseekdatadir):
            print "[Error] Unable to detect imgSeekLib data directory. Make sure imgSeek was installed properly, and report your linux distribution and installation method to nieder at mail dot ru"
            self.imgseekdatadir=sys.exec_prefix+"/share/imgSeek/" # get back to default value
        self.log("imgSeek data dir",self.imgseekdatadir)
        #### check if jpegtran is found
        self.hasJpegtran=binpathfinder("jpegtran")
        if self.hasJpegtran:
            self.log("jpegtran","detected")
        else:
            self.log("jpegtran","not found")
        
    def log(self,who,what):
        """ will wrap any output message during this env init """
        print "[%s] %s" %(who,what)
        
def binpathfinder(prog):
    """ returns the full path for a program binary"""
    ret=None
    pathvar=os.environ["PATH"]
    for pv in split(pathvar,':'):
        if os.path.exists(pv+'/'+prog):
            ret=pv+'/'+prog
            break
        if os.path.exists(pv+prog):
            ret=pv+prog
            break
    return ret
def uniquer(seq, idfun=None):
    """helper function for WriteConfig """
    if idfun is None:
        def idfun(x): return x
    seen = {}
    result = []
    for item in seq:
        marker = idfun(item)
        if marker in seen: continue
        seen[marker] = 1
        result.append(item)
    return result

def LoadConfig(file, config={}):
    """
    returns a dictionary with key's of the form
    <section>.<option> and the values
    """
    config = config.copy()
    cp = ConfigParser.ConfigParser()
    cp.read(file)
    for sec in cp.sections():
        name = lower(sec)
        for opt in cp.options(sec):
            config[name + "." + lower(opt)] = strip(cp.get(sec, opt))
    return config

def WriteConfig(file, config):
    """
    given a dictionary with key's of the form 'section.option: value'
    write() generates a list of unique section names
    creates sections based that list
    use config.set to add entries to each section
        """
    f= open(file,'w')
    cp = ConfigParser.ConfigParser()
    #a little string hacking because our section names are un-normalized
    #this builds a list of all the sections names
    sectionslst = []
    sections = []
    for k in config.keys():
        sectionslst.append(split(k,".")[0])
    #get unique entries
    sections = uniquer(sectionslst)
    for sec in sections:
        #make the headers
        cp.add_section(sec)
        #for each item in my dictionary
        #it splits the key in two and uses that for the first and second "set" args
        #then it uses the item.value for the 3rd arg
        # from 'section.option:value'
    for k in config.items():
        cp.set(split(k[0],".")[0],split(k[0],".")[1] ,k[1] )
    cp.write(f)
    f.close()
